function menu_callback(action)

% Callback function for the menu options in the spider GUI
% program.

switch action

% File menu options
case 'FileLoad'
    % Open file
    [filename, pathname] = uigetfile('*.txt', 'Load Data File');
    if isequal(filename,0)|isequal(pathname,0)
       % disp('File not found')
    else
        loadFile(filename);
    end
    
case 'FileSave'
	vars = get(gcf, 'userdata');
    [filename, pathname] = uiputfile('*.txt', 'Save Data File As...');
    if isequal(filename,0) | isequal(pathname,0)
       % disp('User pressed cancel')
    else
       % disp(['User selected ', fullfile(pathname, filename)])
       % need to write output to file:
       fid = fopen(filename, 'wt');

       for (i=1:length(vars.names)) 
			vars.names{1,i};
			vars.values{1,i};

            line = sprintf('%s %s\n', vars.names{1,i},...
                num2str(vars.values{1,i}));
            fwrite(fid, line);       
       end
       fclose(fid);
    end
	
 
case 'FileExit'
%    Hndl = findobj('Name', 'Spider');
%        close(Hndl);
    close(gcbf);

% View menu options
case 'ViewSpeciesDuplex'
    loadFile('duplex.txt');
case 'ViewSpeciesUetzi'
    loadFile('uetzi.txt');
case 'ViewSpeciesStridulans'
    loadFile('stridulans.txt');
case 'ViewSpeciesCrassipes'
    loadFile('crassipes.txt');

% Edit menu options
case 'EditBodyColor'
    % Slider opens to set RGB colors
    spider_gui('color_slider');
case 'EditLegsColor'
    spider_gui('color_legs_slider');
case 'EditAmbienceLighting'
    spider_gui('ambience_color_slider');
case 'EditDiffuseLighting'
    spider_gui('diffuse_color_slider');
case 'EditSpecLighting'
    spider_gui('spec_color_slider');
case 'EditBackground'
    spider_gui('bg_color_slider');
case 'EditAdvanced'
    spider_gui('edit_advanced');
    
% Play menu options
case 'PlayPlay'
    vars = get(figure(1), 'userdata');
    vars.values{1,153} = vars.tmax*vars.FPS;
    set(figure(1), 'userdata', vars);
    % fig_name = get(figure(1), 'name');
    output = cmex(vars.values{1,:});
case 'PlayStop'
    vars = get(figure(1), 'userdata');
    vars.values{1,153} = 0;
    set(figure(1), 'userdata', vars);
    % fig_name = get(figure(1), 'name');
    output = cmex(vars.values{1,:});

% Callbacks from the 'Edit' Menu options: 
% 'Edit->Color'

case 'Red'
    changeColor('Red');
case 'Green'
    changeColor('Green');
case 'Blue'
    changeColor('Blue');
case 'Alpha'
    
    changeColor('Alpha');
case 'ApplyColor' 
    % change entire spider color at once:
    apply_color(1,2,3,0);
    close(figure(2));
    vars = get(figure(1), 'userdata');
    output = cmex(vars.values{1,:});
case 'ApplyLegsColor'
    apply_color(69,70,71,0);
    apply_color(72,73,74,0);
    apply_color(75,76,77,0);
    apply_color(78,79,80,0);
    apply_color(93,94,95,0);
    apply_color(96,97,98,0);
    apply_color(99,100,101,0);
    apply_color(102,103,104,0);
    apply_color(117,118,119,0);
    apply_color(120,121,122,0);
    apply_color(123,124,125,0);
    apply_color(126,127,128,0);
    apply_color(141,142,143,0);
    apply_color(144,145,146,0);
    apply_color(147,148,149,0);
    apply_color(150,151,152,0);
    close(figure(2));
    vars = get(figure(1), 'userdata');
    output = cmex(vars.values{1,:});
case 'CancelColor'
    close(gcf);
case 'ApplyAmbienceColor'
    apply_color(4, 5, 6, 7);
    close(figure(2));
    vars = get(figure(1), 'userdata');
    output = cmex(vars.values{1,:});
case 'ApplyDiffuseColor'
    apply_color(8, 9, 10, 11);
    close(figure(2));
    vars = get(figure(1), 'userdata');
    output = cmex(vars.values{1,:});
case 'ApplySpecularColor'
    apply_color(12, 13, 14, 15);
    close(figure(2));
    vars = get(figure(1), 'userdata');
    output = cmex(vars.values{1,:});
    
% 'Edit->Background'
case 'ApplyBackgroundColor'
    apply_color(16, 17, 18, 0);
    close(figure(2));
    vars = get(figure(1), 'userdata');
    output = cmex(vars.values{1,:});

% Callback from Main Window: Listbox Selection in upper left corner
% morphology parameters
case 'ChooseParameter'
    Hndl = findobj(figure(1), 'Tag', 'Listbox1');
    value = get(Hndl, 'Value');
    switch value
    case 1
        spider_gui('build_leg1_text_boxes');
    case 2
        spider_gui('build_leg2_text_boxes');
    case 3
        spider_gui('build_leg3_text_boxes');
    case 4
        spider_gui('build_leg4_text_boxes');
    case 5
        % build gui for text boxes to ask for length, width and height
        spider_gui('build_head_text_boxes');
    case 6
        spider_gui('build_thorax_text_boxes');
    case 7
        spider_gui('build_abdomen_text_boxes');
    case 8
        spider_gui('build_spinnerets_text_boxes');
    end

% Callback from Edit->Advanced
% Applies the advance parameter setting to the image
case 'ApplyAdvancedParameter'
    vars = get(figure(1), 'userdata');
    Hndl = findobj(figure(2), 'Tag', 'Listbox1');
    paramValue = get(Hndl, 'Value');
    % the option number corresponds with the parameter settings in vars.values
    debugValue = vars.values{1, paramValue};
    Hndl = findobj(figure(2), 'Tag', 'TextBox1');
    value = get(Hndl, 'String');
    vars.values{1, paramValue} = str2num(value);
    close(figure(2));
    set(figure(1), 'userdata', vars);
    output = cmex(vars.values{1,:});

% Callback from Edit->Advanced
% changes the value seen in the advance parameter setting area:
case 'ChooseAdvancedParameter'
    vars = get(figure(1), 'userdata');
    Hndl = findobj(figure(2), 'Tag', 'Listbox1');
    paramValue = get(Hndl, 'Value');
    % the option number corresponds with the parameter settings in vars.values
    debugValue = vars.values{1, paramValue}
    Hndl = findobj(figure(2), 'Tag', 'TextBox1');
    value = get(Hndl, 'String');
    set(Hndl, 'String', debugValue);

case 'ApplyHeadParams'
    applyMorph(54, 55, 56);
case 'ApplyThoraxParams'
    applyMorph(44, 45, 46);
case 'ApplyAbdomenParams'
    applyMorph(33, 34, 35);


case 'RedsegI'
    changeColor(action);
case 'RedsegII'
    changeColor(action);
case 'RedsegIII'
    changeColor(action);
case 'RedsegIV'
    changeColor(action);
case 'GreensegI'
    changeColor(action);
case 'GreensegII'
    changeColor(action);
case 'GreensegIII'
    changeColor(action);
case 'GreensegIV'
    changeColor(action);
case 'BluesegI'
    changeColor(action);
case 'BluesegII'
    changeColor(action);
case 'BluesegIII'
    changeColor(action);
case 'BluesegIV'
    changeColor(action);
case 'ApplyIParams'
    applyLegMorph(57, 58, 59, 60, 61, 62, 63, 64, ...
        69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80);
case 'ApplyIIParams'  
    applyLegMorph(81,82, 83, 84, 85, 86, 87, 88, ...
        93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104);
case 'ApplyIIIParams'
    applyLegMorph(105, 106, 107, 108, 109, 110, 111, 112, ...
        117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128);
case 'ApplyIVParams'
    applyLegMorph(129, 130, 131, 132, 133, 134, 135,136, ...
        141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152);

% Callback from the Main Window: Spline interpolation section
case 'loadMotion'
    vars = get(figure(1),'userdata');
    fname = [get(vars.filename,'string'),'.mat'];
    load(fname, 'y', 'yy')
    vars.y = y;
    vars.yy = yy;
    hold off
    vars.sp = plot(vars.tt,vars.yy(:,vars.splineIndex));
    hold on
    vars.line1 = plot(vars.t,vars.y(:,vars.splineIndex),'or');
    set(gca, 'ylim', [0 vars.AngleRange]);
    set(gcf,'userdata',vars);
    menu_callback('applySpline');
case 'saveMotion'
    vars= get(gcf,'userdata');
    y = vars.y;
    yy = vars.yy;

    fname = [get(vars.filename,'string'),'.mat'];
    save(fname, 'y', 'yy');
case 'applySpline'
    vars = get(figure(1),'userdata');
    % copy all spline ydata points into vars.values array to pass 
    % into mex function
    tmp=cell(1);
    tmp{1,1}=vars.values{1,1};
    for i=2:153
        tmp={tmp{1,:} vars.values{1,i}};
    end
    vars.values=tmp;
    for splineIndex=1:4
        len = length(vars.yy(:,splineIndex));
        % only want the first 153 items in vars.values
    
        for i=1:length(vars.yy(:,splineIndex))
            vars.values = {vars.values{1,:} vars.yy(i,splineIndex)};
        end
    end    
    set(figure(1), 'userdata', vars);
    output = cmex(vars.values{1,:});
    
case 'splinechoose'
    vars = get(gcf,'userdata');
    objhandle=findobj('tag','splinegroup');
    clrindex=find(objhandle ~= gco);
    set(objhandle(clrindex),'value',0);
    set(gco,'value',1);
    vars.splineIndex = str2num(get(gco,'string'));
    hold off
    vars.sp = plot(vars.tt,vars.yy(:,vars.splineIndex));
    hold on
    vars.line1 = plot(vars.t,vars.y(:,vars.splineIndex),'or');
    set(gca, 'ylim', [0 vars.AngleRange]);
    set(gcf,'userdata',vars);
case 'update'
        vars = get(figure(1),'userdata');
        %get the mouse position in graph units
        mouse = get(gca,'currentpoint');
        
        % xaxis
        if ((mouse(1,1) <= vars.tmax) & (mouse(1,2) <= vars.AngleRange) & ...
           (mouse(1,1) >= 0) & (mouse(1,2) >= 0)) 
            
            %calculate the point nearest to the
            %mouse click
            lineX = get(vars.line1,'xdata');
            lineY = get(vars.line1,'ydata')';
            [val, pnt] = min(abs(lineX-mouse(1,1)));

            %move the control point on the plot
            %and update control point vars
            lineY(pnt) = mouse(1,2);
            set(vars.line1, 'ydata', lineY);
            vars.y(:,vars.splineIndex) = lineY;

            %make new spline
            if vars.ForceZeroSlope
                cs = spline(vars.t,[0; lineY; 0]);
            else
                cs = spline(vars.t,lineY);
            end
            vars.yy(:,vars.splineIndex) = ppval(cs,vars.tt)';
            set(vars.sp, 'ydata', vars.yy(:,vars.splineIndex));
            set(gcf,'userdata',vars);
	    %	output = cmex(vars.values, vars.y(:,vars.splineIndex));
        end

end

% -------------------- Subroutines begin here -----------------------------
% param* specifies parameter settings for opengl
function applyLegMorph(param1, param2, param3, param4, param5, param6, ...
    param7, param8, param9, param10, param11, param12, param13, param14, ...
    param15, param16, param17, param18, param19, param20)
    vars = get(figure(1), 'userdata');
    Hndl = findobj('Tag', 'length');
    lengthParam = get(Hndl, 'String');
    vars.values{1,param1} = str2double(lengthParam(4,:));
    vars.values{1,param2} = str2double(lengthParam(3,:));
    vars.values{1,param3} = str2double(lengthParam(2,:));
    vars.values{1,param4} = str2double(lengthParam(1,:));
    Hndl = findobj('Tag', 'radius');
    radius = get(Hndl, 'String');
    vars.values{1,param5} = str2double(radius(4,:));
    vars.values{1,param6} = str2double(radius(3,:));
    vars.values{1,param7} = str2double(radius(2,:));
    vars.values{1,param8} = str2double(radius(1,:));
    set(figure(1), 'userdata', vars);

    applyLegColor('segI', param9,param10,param11);
    applyLegColor('segII', param12, param13, param14);
    applyLegColor('segIII', param15, param16, param17);
    applyLegColor('segIV', param18, param19, param20);
    vars = get(figure(1), 'userdata');
    output = cmex(vars.values{1,:});

% end applyLegMorph

function applyMorph(param1, param2, param3)
    vars = get(figure(1), 'userdata');
    Hndl = findobj('Tag', 'width');
    headwidth = get(Hndl, 'String');
    vars.values{1,param1} = str2num(headwidth);
    Hndl = findobj('Tag', 'height');
    headheight = get(Hndl, 'String');
    vars.values{1,param2} = str2num(headheight);
    Hndl = findobj('Tag', 'length');
    headlength = get(Hndl, 'String');
    vars.values{1,param3} = str2num(headlength);
    set(figure(1), 'userdata', vars);
    output = cmex(vars.values{1,:});
% end applyMorph

function apply_color(param1, param2, param3, param4)
    vars = get(figure(1), 'userdata');
    Hndl = findobj(gcbf, 'Tag', 'Red');
    vars.values{1,param1} = str2num(get(Hndl, 'String'));
    Hndl = findobj(gcbf, 'Tag', 'Green');
    vars.values{1,param2} = str2num(get(Hndl, 'String'));
    Hndl = findobj(gcbf, 'Tag', 'Blue');
    vars.values{1,param3} = str2num(get(Hndl, 'String'));
    if (param4 ~= 0)
        Hndl = findobj(gcbf, 'Tag', 'Alpha');
        vars.values{1,param4} = str2num(get(Hndl, 'String'));
    end
    set(figure(1), 'userdata', vars);
% end apply_color

function applyLegColor(seg, param1, param2, param3)
    vars = get(figure(1), 'userdata');
    Hndl = findobj(gcbf, 'Tag', ['Red', seg]);
    red = str2num(get(Hndl, 'String'));
    vars.values{1,param1} = red;
    
    Hndl = findobj(gcbf, 'Tag', ['Green', seg]);
    green = str2num(get(Hndl, 'String'));
    vars.values{1,param2} = green;
    
    Hndl = findobj(gcbf, 'Tag', ['Blue', seg]);
    blue = str2num(get(Hndl, 'String'));
    vars.values{1,param3} = blue;
    set(figure(1), 'userdata', vars);

function changeColor(name)
    value = get(gcbo, 'Value');
    Hndl = findobj(gcbf, 'Tag', name);
    str = sprintf('%.2f', value);
    set(Hndl, 'String', str);
% end changeColor

function loadFile(filename)
       vars = get(figure(1), 'userdata');
    
       % disp(['File ', pathname, filename, ' found'])
       fid = fopen(filename);
       
       % Tokenizing allows skipping of commented lines     
       count = 1;
       while (feof(fid) == 0)
          txtline = fgetl(fid);
          
          % Check if this is a commented line.  Note: does not allow commenting to occur
          % in the middle of a line
          comment = findstr(txtline, '%');
          if (isempty(comment) ~= 1)
              continue;
          end
          
          % how do you do reg exps?
		  if (isempty(deblank(txtline)))
			continue;
		  end
          
          
          % we expect exactly 2 fields
          while (isempty(txtline) ~= 1)
              [variable, txtline] = strtok(txtline);
              [value, txtline] = strtok(txtline);

	          % cell arrays:  
			  % how come strtokenizing can't have white spaces at the end of lines?
			  % seems very odd if it's tokenizing according to the space delimiter
              vars.values{1, count} = str2num(value);
              vars.names{1, count} = variable;
			  
          end
          count = count + 1;
       end
       fclose(fid);
       vars.values{1,153} = vars.tmax*vars.FPS;
       set(gcf, 'userdata', vars);
       output = cmex(vars.values{1,:});
%end loadFile